import { TranslateService, TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { CommonModule, DatePipe } from '@angular/common';
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { SplashScreen } from '@ionic-native/splash-screen/ngx';
import { WebStorageStateStore } from 'oidc-client';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { HttpClient, HTTP_INTERCEPTORS } from '@angular/common/http';
import { Camera } from '@ionic-native/camera/ngx';
import { File } from '@ionic-native/file/ngx';
import { Geolocation } from '@ionic-native/geolocation/ngx';
import { HTTP } from '@ionic-native/http/ngx';
import { ImagePicker, OutputType } from '@ionic-native/image-picker/ngx';
import { QRScanner } from '@ionic-native/qr-scanner/ngx';
import { SpeechRecognition } from '@ionic-native/speech-recognition/ngx';
import { SQLite } from '@ionic-native/sqlite/ngx';
import { ReplaySubject, defer, Subject, forkJoin, of } from 'rxjs';
import { Storage } from '@ionic/storage';
import { ToastController, AlertController, LoadingController, Platform, NavParams, Events } from '@ionic/angular';
import { map, take, delay } from 'rxjs/operators';
import { Network } from '@ionic-native/network/ngx';
import { __awaiter } from 'tslib';
import { CriticalError, I18nBaseService, AbstractShowMessage, ShowMessageType, ShowMessageLinkActionType, MessageDisplayerService, LoadingDefault, ApiDateTimeService, HttpInterceptorBaseService, MessageUtils, DefaultDateViewConverterService, DateViewConverter, DateTimeModule, TimeViewConverter, DefaultTimeViewConverterService, UtilsFn, AuthBaseService, AUTH_SETTINGS, AUTH_TYPE, AuthHandlerType, AuthInterceptorHandlerService, UserPermissionService, BaseMessage, SqlUtilsFn } from 'totvs-log-base-foundation';
import { Injectable, Inject, InjectionToken, NgModule, Pipe, APP_INITIALIZER, Injector } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço utilizado para carregar as literais
 */
class IntlLiteralsLoaderService {
    /**
     * Método construtor da classe
     * translate
     * @param {?} translate
     */
    constructor(translate) {
        this.translate = translate;
        /**
         * Responsável por receber as configurações do serviço de tradução para carregar as literais
         */
        this.mobileOptions = { language: 'pt-br', url: '' };
    }
    /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * options
     * @param {?} options
     * @return {?}
     */
    updateOptions(options) {
        try {
            Object.assign(this.mobileOptions, options);
            return this.load();
        }
        catch (error) {
            throw new CriticalError(error);
        }
    }
    /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     * @return {?}
     */
    load() {
        return this.translate.getTranslation(this.mobileOptions.language)
            .pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        (res) => res)));
    }
}
IntlLiteralsLoaderService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
IntlLiteralsLoaderService.ctorParameters = () => [
    { type: TranslateService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Token de configuração da url de internacionalização
 * @type {?}
 */
const INTL_CONFIG = new InjectionToken('Configurações da internacionalização');
/**
 * Serviço de internacionalização da aplicação
 */
class IntlService extends I18nBaseService {
    /**
     * Método construtor que inicializa a classe
     * translate Serviço de internacionalização
     * @param {?} config
     * @param {?} intlLiteralsLoader
     */
    constructor(config, intlLiteralsLoader) {
        super(intlLiteralsLoader);
        /**
         * Atributo de opções de internacionalização
         */
        this.options = { language: 'pt-br', url: '' };
        /** @type {?} */
        let matchedNavigatorAppLanguage;
        if (config.supportedLanguages) {
            matchedNavigatorAppLanguage = config.supportedLanguages.find((/**
             * @param {?} appSupportedLanguage
             * @return {?}
             */
            appSupportedLanguage => {
                return appSupportedLanguage.language === navigator.language;
            }));
        }
        if (!matchedNavigatorAppLanguage) {
            matchedNavigatorAppLanguage = { language: 'pt-br' };
        }
        this.options.language = matchedNavigatorAppLanguage.language.toLowerCase();
    }
}
IntlService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
IntlService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [INTL_CONFIG,] }] },
    { type: IntlLiteralsLoaderService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de mensagens
 */
class ShowMessageService extends AbstractShowMessage {
    /**
     * Método construtor que inicializa a classe
     * toastCtrl Serviço de TOAST
     * alertCtrl Serviço de Alerta
     * i18n Serviço de internacionalização
     * @param {?} toastCtrl
     * @param {?} alertCtrl
     */
    constructor(toastCtrl, alertCtrl) {
        super();
        this.toastCtrl = toastCtrl;
        this.alertCtrl = alertCtrl;
    }
    /**
     * Método responsável por apresentar uma mensagem
     * @param {?} showMessageOptions
     * @return {?}
     */
    displayMessage(showMessageOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (showMessageOptions.messageType) {
                case ShowMessageType.NotificationSuccess:
                    yield this.showNotification(showMessageOptions);
                    break;
                case ShowMessageType.DialogAlert:
                case ShowMessageType.DialogConfirmation:
                    yield this.showDialog(showMessageOptions);
                    break;
                default:
                    throw new Error('Tipo de mensagem inválido');
            }
        });
    }
    /**
     * Método responsável por apresentar uma mensagem de notificação
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    showNotification(showMessageOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const toast = yield this.toastCtrl.create({
                message: showMessageOptions.messageText,
                duration: showMessageOptions.messageLinkActionType ===
                    ShowMessageLinkActionType.No
                    ? 3000
                    : 10000,
                position: 'middle',
                showCloseButton: showMessageOptions.messageLinkActionType !==
                    ShowMessageLinkActionType.No,
                closeButtonText: showMessageOptions.messageLinkLabel
            });
            toast.onDidDismiss().then((/**
             * @return {?}
             */
            () => {
                if (showMessageOptions.messageLinkActionType ===
                    ShowMessageLinkActionType.LinkNewTab) {
                    this.openLink(showMessageOptions.messageLinkAction);
                }
                else if (showMessageOptions.messageLinkActionType ===
                    ShowMessageLinkActionType.Method &&
                    showMessageOptions.messageLinkAction) {
                    showMessageOptions.messageLinkAction();
                }
            }));
            yield toast.present();
        });
    }
    /**
     * Método responsável por apresentar uma mensagem de diálogo
     * @private
     * @param {?} showMessageOptions
     * @return {?}
     */
    showDialog(showMessageOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const buttons = [];
            /** @type {?} */
            let alert;
            if (showMessageOptions.messageCancelLabel &&
                showMessageOptions.messageCancelLabel.trim() !== '' &&
                showMessageOptions.messageType === ShowMessageType.DialogConfirmation) {
                buttons.push({
                    text: showMessageOptions.messageCancelLabel,
                    handler: (/**
                     * @return {?}
                     */
                    () => {
                        alert.dismiss(false);
                        return false;
                    })
                });
            }
            if (showMessageOptions.messageConfirmLabel &&
                showMessageOptions.messageConfirmLabel.trim() !== '') {
                buttons.push({
                    text: showMessageOptions.messageConfirmLabel,
                    handler: (/**
                     * @return {?}
                     */
                    () => {
                        alert.dismiss(true);
                        return false;
                    })
                });
            }
            alert = yield this.alertCtrl.create({
                message: showMessageOptions.messageDetails,
                header: showMessageOptions.messageTitle,
                subHeader: showMessageOptions.messageText,
                buttons
            });
            alert.onDidDismiss().then((/**
             * @param {?} data
             * @return {?}
             */
            data => {
                switch (data) {
                    case true:
                        if (showMessageOptions.messageConfirmAction) {
                            showMessageOptions.messageConfirmAction();
                        }
                        break;
                    default:
                        if (showMessageOptions.messageCancelAction) {
                            showMessageOptions.messageCancelAction();
                        }
                }
            }));
            yield alert.present();
        });
    }
    /**
     * Método responsável pela abertura de um link
     * @private
     * @param {?} link
     * @return {?}
     */
    openLink(link) {
        link = link.indexOf('http://') === -1 ? `http://${link}` : link;
        window.open(link, '_blank');
    }
}
ShowMessageService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ShowMessageService.ctorParameters = () => [
    { type: ToastController },
    { type: AlertController }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ShowMessageModule {
}
ShowMessageModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                exports: [],
                providers: [
                    ShowMessageService,
                    MessageDisplayerService,
                    {
                        provide: AbstractShowMessage,
                        useClass: ShowMessageService
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Pipe de internacionalização da aplicação
 */
class I18NPipe {
    /**
     * Método construtor que inicializa a classe
     * i18nService Serviço de internacionalização
     * @param {?} i18nService
     */
    constructor(i18nService) {
        this.i18nService = i18nService;
    }
    /**
     * Método responsável pela transformação da string
     * query
     * data
     * @param {?} query
     * @param {?} data
     * @return {?}
     */
    transform(query, data) {
        if (!query || query.length === 0) {
            return '';
        }
        /** @type {?} */
        const ret = this.cached(query, data);
        if (ret) {
            return ret;
        }
        if (!data) {
            data = [''];
        }
        else if (!Array.isArray(data)) {
            data = [data];
        }
        return this.i18nService.format(data, query);
    }
    /**
     * Método responsável por retornar o que está em cache
     * query
     * args
     * @private
     * @param {?} query
     * @param {...?} args
     * @return {?}
     */
    cached(query, ...args) {
        return '';
    }
}
I18NPipe.decorators = [
    { type: Pipe, args: [{ name: 'format' },] }
];
/** @nocollapse */
I18NPipe.ctorParameters = () => [
    { type: I18nBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const i18nFactory = (/**
 * @param {?} i18n
 * @param {?} config
 * @return {?}
 */
(i18n, config) => {
    return (/**
     * @return {?}
     */
    () => new Promise((/**
     * @param {?} resolve
     * @return {?}
     */
    resolve => {
        if (config.lazy) {
            resolve(true);
        }
        else {
            i18n.updateOptions({}).subscribe((/**
             * @return {?}
             */
            () => {
                resolve(true);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                console.error(error);
                resolve(false);
            }));
        }
    })));
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Função responsável por indicar o diretório que contém os arquivos de tradução
 * http
 * @param {?} http
 * @return {?}
 */
function createTranslateLoader(http) {
    return new TranslateHttpLoader(http, './assets/i18n/', '.json');
}
const ɵ0 = i18nFactory;
/**
 * Módulo responsável pela configuração da internacionalização da aplicação
 */
class IntlModule {
}
IntlModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    TranslateModule.forRoot({
                        loader: {
                            provide: TranslateLoader,
                            useFactory: createTranslateLoader,
                            deps: [HttpClient],
                        },
                    }),
                ],
                declarations: [I18NPipe],
                exports: [TranslateModule, I18NPipe],
                providers: [
                    IntlLiteralsLoaderService,
                    {
                        provide: I18nBaseService,
                        useClass: IntlService,
                    },
                    {
                        provide: APP_INITIALIZER,
                        useFactory: ɵ0,
                        deps: [I18nBaseService, INTL_CONFIG],
                        multi: true,
                    },
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de loading
 */
class ShowLoadingService extends LoadingDefault {
    /**
     * Método construtor que inicializa a classe
     * i18n Serviço de internacionalização
     * loadingController Serviço de loading
     * @param {?} i18n
     * @param {?} loadingController
     */
    constructor(i18n, loadingController) {
        super();
        this.i18n = i18n;
        this.loadingController = loadingController;
        this.loadingActived = new ReplaySubject(1);
        this.thfContentTemplate = '<span class="thf-loading-icon"></span>{content}';
    }
    /**
     * @return {?}
     */
    showLoading() {
        if (!this.active) {
            this.active = true;
            /** @type {?} */
            let content = '';
            if (this.loadingText) {
                content = this.loadingText.text;
            }
            this.loadingController
                .create({
                message: this.thfContentTemplate.replace('{content}', content),
                spinner: null,
            })
                .then((/**
             * @param {?} loading
             * @return {?}
             */
            loading => {
                this.loading = loading;
                this.loading.present().then((/**
                 * @return {?}
                 */
                () => {
                    this.loadingActived.next();
                }));
            }));
        }
        return this.loadingActived.pipe(take(1));
    }
    /**
     * @return {?}
     */
    hideLoading() {
        this.loadingActived.pipe(take(1)).subscribe((/**
         * @return {?}
         */
        () => {
            this.active = false;
            this.loadingText = undefined;
            this.loading.dismiss();
        }));
    }
    /**
     * @param {?} loadingText
     * @return {?}
     */
    setLoadingText(loadingText) {
        this.loadingActived.pipe(take(1)).subscribe((/**
         * @return {?}
         */
        () => {
            setTimeout((/**
             * @return {?}
             */
            () => {
                /** @type {?} */
                const baseMessage = this.i18n.text(loadingText);
                if (this.loading) {
                    this.loading.message = this.thfContentTemplate.replace('{content}', baseMessage.text);
                    // this.loading.forceUpdate();
                }
                this.loadingText = baseMessage;
            }), 33);
        }));
    }
}
ShowLoadingService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ShowLoadingService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: LoadingController }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável por fornecer atributos e métodos que
 * poderão ser utilizados em toda a aplicação
 */
class GlobalService {
    /**
     * Método construtor que inicializa o serviço
     * i18n Serviço de internacionalização
     * msg Serviço de mensageria para usuário
     * date Serviço de formatação de data
     * @param {?} i18n
     * @param {?} msg
     * @param {?} loading
     * @param {?} date
     */
    constructor(i18n, msg, loading, date) {
        this.i18n = i18n;
        this.msg = msg;
        this.loading = loading;
        this.date = date;
    }
}
GlobalService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
GlobalService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: MessageDisplayerService },
    { type: ShowLoadingService },
    { type: ApiDateTimeService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço utilizado para realizar o tratamento das respostas
 */
class DefaultHttpInterceptorService extends HttpInterceptorBaseService {
    /**
     * Método construtor da classe
     * inj
     * @param {?} inj
     */
    constructor(inj) {
        super();
        this.inj = inj;
    }
    /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * response
     * @param {?} response
     * @return {?}
     */
    interceptResponse(response) {
        this.msg = this.inj.get(MessageDisplayerService);
        if (response.body && response.body.totvs_messages) {
            if (response.body.totvs_messages instanceof Array) {
                /** @type {?} */
                const messages = response.body.totvs_messages;
                for (const message of messages) {
                    this.showTotvsMessage(message);
                }
            }
            else {
                this.showTotvsMessage((/** @type {?} */ (response.body.totvs_messages)));
            }
        }
    }
    /**
     * Método responsável por realizar o tratamento de respostas de erro
     * response
     * request
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    interceptError(response, request) {
        /** @type {?} */
        const error = (/** @type {?} */ (response.error));
        if (error && error.message && !HttpInterceptorBaseService.isErrorNotificationDisabled(request)) {
            /** @type {?} */
            const title = HttpInterceptorBaseService.titleFromTotvsError(error);
            /** @type {?} */
            const detail = HttpInterceptorBaseService.detailFromTotvsError(error);
            this.showTotvsMessage({
                type: 'error',
                code: title,
                detail
            });
        }
    }
    /**
     * Método responsável por mostrar a mensagem
     * message
     * @param {?} message
     * @return {?}
     */
    showTotvsMessage(message) {
        /** @type {?} */
        const messageType = MessageUtils.showMessageTypeFromTotvsMessage(message);
        this.msg.displayMessage({
            messageType,
            messageText: message.code,
            messageDetails: message.detail
        });
    }
    /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * req
     * next
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    intercept(req, next) {
        return super.intercept(req, next);
    }
}
DefaultHttpInterceptorService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
DefaultHttpInterceptorService.ctorParameters = () => [
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InterceptorModule {
}
InterceptorModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [],
                providers: [
                    DefaultHttpInterceptorService,
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: DefaultHttpInterceptorService,
                        multi: true
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DateTimeModule$1 {
}
DateTimeModule$1.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule,
                    DateTimeModule
                ],
                exports: [],
                providers: [
                    {
                        provide: DateViewConverter,
                        useClass: DefaultDateViewConverterService
                    },
                    {
                        provide: TimeViewConverter,
                        useClass: DefaultTimeViewConverterService
                    },
                    DatePipe,
                    DefaultDateViewConverterService,
                    DefaultTimeViewConverterService,
                    ApiDateTimeService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ShowLoadingModule {
}
ShowLoadingModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                exports: [],
                providers: [
                    LoadingController,
                    ShowLoadingService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FndModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: FndModule,
            providers: [GlobalService, { provide: INTL_CONFIG, useValue: config }],
        };
    }
}
FndModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    IntlModule,
                    ShowMessageModule,
                    InterceptorModule,
                    DateTimeModule$1,
                    ShowLoadingModule,
                ],
                declarations: [],
                exports: [
                    IntlModule,
                    ShowMessageModule,
                    InterceptorModule,
                    DateTimeModule$1,
                    ShowLoadingModule,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Esta classe simula a classe Platform e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: Platform, useClass: PlatformMock},
 *   ...
 * ]
 * ```
 */
class PlatformMock {
    /**
     * Método que simula a ação de preparado da plataforma
     * @return {?}
     */
    ready() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            resolve('READY');
        }));
    }
    /**
     * Método que simula o retorno dos parâmetros de query de URL
     * @return {?}
     */
    getQueryParam() {
        return true;
    }
    /**
     * Método que simula o registro da ação para o botão 'Voltar'
     * fn
     * priority
     * @param {?} fn
     * @param {?=} priority
     * @return {?}
     */
    registerBackButtonAction(fn, priority) {
        return ((/**
         * @return {?}
         */
        () => true));
    }
    /**
     * Método que simula a verificação de foco
     * ele
     * @param {?} ele
     * @return {?}
     */
    hasFocus(ele) {
        return true;
    }
    /**
     * Método que simula o retorno do 'document'
     * @return {?}
     */
    doc() {
        return document;
    }
    /**
     * Método que simula o retorno do 'is'
     * @return {?}
     */
    is() {
        return true;
    }
    /**
     * Método que simula o retorno do estilo de um elemento
     * container
     * @param {?} container
     * @return {?}
     */
    getElementComputedStyle(container) {
        return {
            paddingLeft: '10',
            paddingTop: '10',
            paddingRight: '10',
            paddingBottom: '10',
        };
    }
    /**
     * Método que simula o evento de redimensionamento de tela
     * callback
     * @param {?} callback
     * @return {?}
     */
    onResize(callback) {
        return callback;
    }
    /**
     * Método que simula o registro de evento de escuta
     * ele
     * eventName
     * callback
     * @param {?} ele
     * @param {?} eventName
     * @param {?} callback
     * @return {?}
     */
    registerListener(ele, eventName, callback) {
        return ((/**
         * @return {?}
         */
        () => true));
    }
    /**
     * Método que simula o retorno do método 'win'
     * @return {?}
     */
    win() {
        return window;
    }
    /**
     * Método que simula o retorno do método 'raf'
     * callback
     * @param {?} callback
     * @return {?}
     */
    raf(callback) {
        return 1;
    }
    /**
     * Método que simula a definição de tempo de espera
     * callback
     * timer
     * @param {?} callback
     * @param {?} timer
     * @return {?}
     */
    timeout(callback, timer) {
        return setTimeout(callback, timer);
    }
    /**
     * Método que simula o cancelamento do tempo de espera
     * id
     * @param {?} id
     * @return {?}
     */
    cancelTimeout(id) {
        // do nothing
    }
    /**
     * Método que simula o retorno do elemento ativo
     * @return {?}
     */
    getActiveElement() {
        return document.activeElement;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Esta classe simula a classe StatusBar e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: StatusBar, useClass: StatusBarMock},
 *   ...
 * ]
 * ```
 */
class StatusBarMock extends StatusBar {
    /**
     * Método que simula o retorno do estilo padrão
     * @return {?}
     */
    styleDefault() {
        return;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Esta classe simula a classe SplashScreen e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: SplashScreen, useClass: SplashScreenMock},
 *   ...
 * ]
 * ```
 */
class SplashScreenMock extends SplashScreen {
    /**
     * Método que simula o evento para esconder o 'splash'
     * @return {?}
     */
    hide() {
        return;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Esta classe simula a classe NavParams e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: NavParams, useClass: NavParamsMock},
 *   ...
 * ]
 * ```
 */
class NavParamsMock {
    /**
     * Método que simula a definição de parâmetros
     * value
     * @param {?} value
     * @return {?}
     */
    static setParams(value) {
        NavParamsMock.returnParam = value;
    }
    /**
     * Método que simula o retorno do valor de uma chave
     * key
     * @param {?} key
     * @return {?}
     */
    get(key) {
        if (NavParamsMock.returnParam) {
            return NavParamsMock.returnParam;
        }
        return 'default';
    }
}
/**
 * Parâmetro de retorno
 */
NavParamsMock.returnParam = null;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém serviços simulados necessários para o desenvolvimento
 * de testes unitários automatizados utilizando ionic
 */
class MockModule {
}
MockModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [],
                providers: [
                    // { provide: DeepLinker, useClass: DeepLinkerMock},
                    { provide: NavParams, useClass: NavParamsMock },
                    // { provide: Nav, useClass: NavMock },
                    { provide: Platform, useClass: PlatformMock },
                    { provide: SplashScreen, useClass: SplashScreenMock },
                    { provide: StatusBar, useClass: StatusBarMock }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Create async observable that emits-once and completes
 *  after a JS engine turn
 * @template T
 * @param {?} data
 * @return {?}
 */
function asyncData(data) {
    return defer((/**
     * @return {?}
     */
    () => Promise.resolve(data)));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Aguardar o espaço de tempo de um quadro na quantidade de quadros/seg
 * desejada para que o ionic renderize corretamente os componentes em tela
 * \@example
 * it('Deveria renderizar', async () => {
 *   await component.ngOnInit();
 *   await render(fixture);
 *   expect(fixture.nativeElement.querySelector('#id').value).toBe('Valor');
 * });
 *
 * @template T
 * @param {?} fixture Referência do componente de teste
 * @param {?=} fps Quantidade de quadros por segundo para referência
 * @return {?}
 */
function render(fixture, fps = 30) {
    return __awaiter(this, void 0, void 0, function* () {
        fixture.detectChanges();
        yield fixture.whenRenderingDone();
        yield fixture.whenStable();
        yield UtilsFn.sleep(1000 / fps);
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo Popup utilizado no processo
 * de autenticação com Oidc no Ionic
 */
class OidcCordovaPopupWindow {
    /**
     * Método construtor que inicializa a classe
     * iab Serviço do plugin nativo ionic InAppBrowser
     * popupParams Parâmetros para o popup
     * @param {?} iab
     * @param {?} popupParams
     */
    constructor(iab, popupParams) {
        this.iab = iab;
        this.popupParams = popupParams;
        this.navigated = new Subject();
        this.redirectUri = this.popupParams.startUrl;
    }
    /**
     * Método acessor do promise retornado pelo método Navigate
     * @return {?}
     */
    get promise() {
        return this.internalPromise;
    }
    /**
     * Método responsável por realizar a navegação
     * navParams Parâmetros de navegação
     * @param {?} navParams
     * @return {?}
     */
    navigate(navParams) {
        this.internalPromise = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            this.internalReject = reject;
            /** @type {?} */
            let errorMsg = '';
            if (!navParams || !navParams.url) {
                errorMsg = 'No url provided';
            }
            else {
                /** @type {?} */
                const features = this.popupParams.popupWindowFeatures || OidcCordovaPopupWindow.DefaultPopupFeatures;
                /** @type {?} */
                const target = this.popupParams.popupWindowTarget || OidcCordovaPopupWindow.DefaultPopupTarget;
                this.popup = this.iab.create(navParams.url, target, features);
                if (this.popup) {
                    // Evento 'exit'
                    this.popupExitSub = this.popup.on('exit').subscribe((/**
                     * @param {?} event
                     * @return {?}
                     */
                    (event) => {
                        this.closePopup();
                        reject(new Error(event.message));
                    }));
                    // Evento 'loadstart'
                    this.popupLoadStartSub = this.popup.on('loadstart').subscribe((/**
                     * @param {?} event
                     * @return {?}
                     */
                    (event) => {
                        /** @type {?} */
                        const url = event.url;
                        if (url.indexOf(this.redirectUri) === 0) {
                            this.closePopup();
                            resolve({ url });
                        }
                        else if (url.indexOf('error') !== -1) {
                            this.closePopup();
                            reject(new Error(`Load start error. URL: ${url}`));
                        }
                    }));
                }
                else {
                    errorMsg = 'Error opening popup window';
                }
            }
            if (errorMsg !== '') {
                this.closePopup();
                reject(new Error('Error opening popup window'));
            }
        }));
        this.navigated.next(this.promise);
        return this.promise;
    }
    /**
     * Método responsável por fechar
     * @return {?}
     */
    close() {
        this.closePopup();
        this.internalReject();
    }
    /**
     * Método responsável por fechar o popup
     * @private
     * @return {?}
     */
    closePopup() {
        if (this.popup) {
            this.popupExitSub.unsubscribe();
            this.popupLoadStartSub.unsubscribe();
            this.popup.close();
            this.popup = undefined;
        }
    }
}
/**
 * Propriedades padrão do Popup
 */
OidcCordovaPopupWindow.DefaultPopupFeatures = 'location=no,toolbar=no,zoom=no';
/**
 * Destino padrão do Popup
 */
OidcCordovaPopupWindow.DefaultPopupTarget = '_blank';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo Iframe utilizado no processo
 * de autenticação com Oidc no Ionic
 */
class OidcCordovaIFrameNavigator {
    /**
     * Método construtor que inicializa a classe
     * iab Serviço do plugin nativo InAppBrowser
     * @param {?} iab
     */
    constructor(iab) {
        this.iab = iab;
    }
    /**
     * Método que prepara a janela de popup
     * params Parâmetros para a janela de popup
     * @param {?} params
     * @return {?}
     */
    prepare(params) {
        params.popupWindowFeatures = 'hidden=yes';
        params.isIframe = true;
        /** @type {?} */
        const popup = new OidcCordovaPopupWindow(this.iab, params);
        return Promise.resolve(popup);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OidcCordovaIabEvents {
}
OidcCordovaIabEvents.LoadSuccessful = 'OidcCordovaIabEventsLoadSuccessful';
OidcCordovaIabEvents.LoadFailed = 'OidcCordovaIabEventsLoadFailed';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pela navegação com popup utilizado no processo
 * de autenticação com Oidc no Ionic
 */
class OidcCordovaPopupNavigator {
    /**
     * Método construtor que inicializa a classe
     * iab Serviço do plugin nativo InAppBrowser
     * @param {?} iab
     * @param {?} events
     */
    constructor(iab, events) {
        this.iab = iab;
        this.events = events;
    }
    /**
     * Método que prepara a janela de popup
     * params Parâmetros para a janela de popup
     * @param {?} params
     * @return {?}
     */
    prepare(params) {
        /** @type {?} */
        const popup = new OidcCordovaPopupWindow(this.iab, params);
        popup.navigated.pipe(take(1)).subscribe((/**
         * @param {?} promise
         * @return {?}
         */
        promise => {
            promise
                .then((/**
             * @param {?} value
             * @return {?}
             */
            value => {
                this.events.publish(OidcCordovaIabEvents.LoadSuccessful);
                return value;
            }))
                .catch((/**
             * @param {?} err
             * @return {?}
             */
            err => {
                this.events.publish(OidcCordovaIabEvents.LoadFailed);
                throw err;
            }));
        }));
        return Promise.resolve(popup);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe de serviço com métodos para autenticação
 * @extends AuthBaseService
 */
class AuthService extends AuthBaseService {
    /**
     * Método construtor da classe
     * authSettings Token de injeção com as configurações de autenticação
     * authType Token de injeção com o tipo de autenticação
     * @param {?} authSettings
     * @param {?} authType
     * @param {?} events
     */
    constructor(authSettings, authType, events) {
        super(authSettings);
        this.events = events;
        this.authType = authType;
    }
    /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    loginFlow() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.authType.type === AuthHandlerType.oauth2) {
                /** @type {?} */
                let loaded;
                /** @type {?} */
                let doneFn;
                /** @type {?} */
                let loadedFn;
                /** @type {?} */
                let failedFn;
                /** @type {?} */
                const doneProm = new Promise((/**
                 * @param {?} resolve
                 * @param {?} reject
                 * @return {?}
                 */
                (resolve, reject) => {
                    doneFn = resolve;
                }));
                try {
                    loadedFn = (/**
                     * @return {?}
                     */
                    () => {
                        console.log('loadedFn');
                        loaded = true;
                        UtilsFn.timeout(doneProm, 1000);
                        this.events.unsubscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn);
                    });
                    this.events.subscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn);
                    failedFn = (/**
                     * @return {?}
                     */
                    () => {
                        console.log('failedFn');
                        loaded = false;
                        UtilsFn.timeout(doneProm, 1000);
                        this.events.unsubscribe(OidcCordovaIabEvents.LoadFailed, failedFn);
                    });
                    this.events.subscribe(OidcCordovaIabEvents.LoadFailed, failedFn);
                    yield this.manager.signinPopup();
                    doneFn();
                }
                catch (error) {
                    console.log('catch');
                    doneFn();
                    this.events.unsubscribe(OidcCordovaIabEvents.LoadFailed, failedFn);
                    this.events.unsubscribe(OidcCordovaIabEvents.LoadSuccessful, loadedFn);
                    console.log('error', error);
                    if (!loaded || error) {
                        throw error;
                    }
                }
            }
        });
    }
    /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    logoutFlow() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.authType.type === AuthHandlerType.oauth2) {
                yield this.manager.signoutPopup();
            }
        });
    }
}
AuthService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_SETTINGS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_TYPE,] }] },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} iab
 * @param {?} events
 * @param {?} authSettings
 * @return {?}
 */
function authFactory(iab, events, authSettings) {
    return Object.assign(authSettings, {
        popupNavigator: new OidcCordovaPopupNavigator(iab, events),
        iframeNavigator: new OidcCordovaIFrameNavigator(iab),
        checkSessionInterval: authSettings.checkSessionInterval || 30000,
        userStore: new WebStorageStateStore({ store: window.localStorage }),
    });
}
/** @type {?} */
const AUTH_FACTORY = new InjectionToken('');
/**
 * Módulo que exporta e ativa as dependências para utilizar autenticação com fluxo oauth2
 * Para garantir que os serviços tenham uma única instância, usar AuthModule.forRoot() ao importar
 * @export
 * FndModule
 */
class AuthModule {
    /**
     * \@dynamic
     * @param {?} authSettings
     * @param {?} handlerTypeSettings
     * @return {?}
     */
    static forRoot(authSettings, handlerTypeSettings) {
        return {
            ngModule: AuthModule,
            providers: [
                InAppBrowser,
                UserPermissionService,
                {
                    provide: AuthBaseService,
                    useClass: AuthService,
                },
                {
                    provide: AUTH_FACTORY,
                    useValue: authSettings
                },
                {
                    provide: AUTH_SETTINGS,
                    useFactory: authFactory,
                    deps: [InAppBrowser, Events, AUTH_FACTORY],
                },
                { provide: AUTH_TYPE, useValue: handlerTypeSettings },
            ],
        };
    }
}
AuthModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: AuthInterceptorHandlerService,
                        multi: true,
                    },
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém todos os eventos relacionados ao Provider de Tenant
 */
class MultiTenantEvents {
}
/**
 * Evento disparado quando é realizado uma mudança do tenant corrente e este é diferente do anterior
 */
MultiTenantEvents.CURRENT_TENANT_CHANGED = 'CurrentTenant:Changed';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
class CameraUtilEvents {
}
// FIXME - Nomes de eventos mais específicos, para evitar possíveis conflitos
/**
 * Evento publicado pelo utilitário quando uma foto é tirada e o arquivo é gerado
 * no cache do aplicativo
 */
CameraUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de camera
 * com métodos que facilitam a solicitação de permissão e apresentação da
 * camera
 */
class CameraUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * camera Serviço do plugin nativo de camera
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} camera
     * @param {?} global
     * @param {?} events
     */
    constructor(platform, camera, global, events) {
        this.camera = camera;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => {
            this.isCordova = platform.is('cordova');
            if (this.isCordova) {
                this.defaultOptions = {
                    quality: 75,
                    destinationType: this.camera.DestinationType.FILE_URI,
                    encodingType: this.camera.EncodingType.JPEG,
                    mediaType: this.camera.MediaType.PICTURE,
                    targetWidth: 1920,
                    targetHeight: 1080,
                    correctOrientation: true
                };
            }
        }));
    }
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * {CameraOptions} options Opções da camera
     * @param {?=} options
     * @return {?} Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    takePicture(options) {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (this.isCordova === false) {
                this.global.msg.showMessage(new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                }));
                reject();
            }
            /** @type {?} */
            const cameraOptions = Object.assign({}, this.defaultOptions);
            if (options) {
                for (const prop of Object.keys(options)) {
                    cameraOptions[prop] = options[prop];
                }
            }
            this.camera.getPicture(cameraOptions)
                .then((/**
             * @param {?} imageData
             * @return {?}
             */
            (imageData) => {
                this.events.publish(CameraUtilEvents.CACHE_FILE_CREATED, imageData);
                resolve(imageData);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => reject(error))).catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                /** @type {?} */
                let literal;
                if (error && error === 20) {
                    literal = 'permissaoNaoConcedidaParaORecursoMsg';
                }
                else {
                    literal = 'recursoNaoEstaDisponivelMsg';
                }
                this.global.msg.showMessage(new BaseMessage({
                    literal
                }));
                reject(error);
            }));
        }));
    }
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     * @return {?}
     */
    cleanup() {
        if (this.isCordova) {
            this.camera.cleanup()
                .then((/**
             * @return {?}
             */
            () => { }))
                .catch((/**
             * @return {?}
             */
            () => { }));
        }
    }
}
CameraUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
CameraUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: Camera },
    { type: GlobalService },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de camera
 */
class CameraUtilModule {
}
CameraUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    Camera,
                    CameraUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de arquivo
 * com métodos que facilitam a manipulação de arquivos no aplicativo
 */
class FileUtilProvider {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     * @param {?} platform
     * @param {?} file
     */
    constructor(platform, file) {
        this.file = file;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por remover um arquivo
     * uri URI do arquivo que deve ser removido
     * Objeto que representa o resultado da remoção do arquivo
     * @param {?} uri
     * @return {?}
     */
    removeFile(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                throw new CriticalError('Platform is not cordova (removeFile)');
            }
            if (!uri || uri === '') {
                throw new CriticalError('URI invalid (removeFile)');
            }
            /** @type {?} */
            const fileName = this.getFilenameFromUri(uri);
            /** @type {?} */
            const path = this.getPathFromUri(uri);
            /** @type {?} */
            let removeResult;
            try {
                removeResult = yield this.file.removeFile(path, fileName);
            }
            catch (error) {
                throw new CriticalError('Error in remove file', error);
            }
            return removeResult;
        });
    }
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * uris URIs dos arquivos que devem ser copiados
     * Lista de URIs dos arquivos copiados para o storage do aplicativo
     * @param {...?} uris
     * @return {?}
     */
    fileCacheToAppStorage(...uris) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                throw new CriticalError('Platform is not cordova');
            }
            /** @type {?} */
            const promisesCopy = [];
            uris.forEach((/**
             * @param {?} uri
             * @return {?}
             */
            (uri) => {
                /** @type {?} */
                const fileName = decodeURI(this.getFilenameFromUri(uri));
                /** @type {?} */
                const path = this.getPathFromUri(uri);
                promisesCopy.push(this.file.copyFile(path, fileName, this.file.dataDirectory, fileName)
                    .then((/**
                 * @param {?} entry
                 * @return {?}
                 */
                (entry) => {
                    return {
                        sourcePath: path,
                        sourceFileName: fileName,
                        targetFile: entry
                    };
                })));
            }));
            /** @type {?} */
            let filesCopied;
            try {
                filesCopied = yield Promise.all(promisesCopy);
            }
            catch (error) {
                throw new CriticalError(`Error in copy file`, error);
            }
            /** @type {?} */
            const promisesRemove = [];
            /** @type {?} */
            const filesUrl = [];
            filesCopied.forEach((/**
             * @param {?} fileCopied
             * @return {?}
             */
            (fileCopied) => {
                filesUrl.push(fileCopied.targetFile.toURL());
                promisesRemove.push(this.file.removeFile(fileCopied.sourcePath, fileCopied.sourceFileName)
                    .then((/**
                 * @param {?} result
                 * @return {?}
                 */
                (result) => {
                    return result;
                })));
            }));
            try {
                yield Promise.all(promisesRemove);
            }
            catch (error) {
                throw new CriticalError(`Error in remove file`, error);
            }
            return filesUrl;
        });
    }
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    getFilenameFromUri(uri) {
        /** @type {?} */
        let fileName;
        try {
            /** @type {?} */
            const fileNameArray = uri.split('/');
            fileName = fileNameArray[fileNameArray.length - 1];
        }
        catch (error) {
            throw new CriticalError('Error (getFilenameFromUri)', error);
        }
        return fileName;
    }
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * uri URI do arquivo
     * @private
     * @param {?} uri
     * @return {?}
     */
    getPathFromUri(uri) {
        /** @type {?} */
        let filePath;
        try {
            /** @type {?} */
            const fileNameArray = uri.split('/');
            fileNameArray.splice(fileNameArray.length - 1, 1);
            filePath = fileNameArray.join('/');
        }
        catch (error) {
            throw new CriticalError('Error (getPathFromUri)', error);
        }
        return filePath;
    }
}
FileUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
FileUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: File }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de manipulação de arquivo
 */
class FileUtilModule {
}
FileUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    File,
                    FileUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de geolocalização
 * com métodos que facilitam a solicitação de permissão e identificação de geolocalização
 */
class GeolocationUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * geolocation Serviço do plugin nativo de geolocalização
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} geolocation
     * @param {?} global
     */
    constructor(platform, geolocation, global) {
        this.geolocation = geolocation;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por obter a posição corrente do usuário
     * {GeolocationOptions} options Opções para recuperação da geolocalização
     * Posição
     * @param {?} options
     * @return {?}
     */
    getCurrentPosition(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                /** @type {?} */
                const msg = new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                });
                this.global.msg.showMessage(msg);
                throw new CriticalError('Platform is not cordova');
            }
            return yield this.geolocation.getCurrentPosition(options);
        });
    }
    /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * point1 Coordenadas do primeiro ponto
     * point2 Coordenadas do segundo ponto
     * @param {?} point1
     * @param {?} point2
     * @return {?}
     */
    calculatePointsKMDistance(point1, point2) {
        if (!this.validGeoPositionPoint(point1) || !this.validGeoPositionPoint(point2)) {
            throw new Error('ERROR - Invalid values');
        }
        if (point1.latitude === point2.latitude && point1.longitude === point2.longitude) {
            return 0;
        }
        /** @type {?} */
        const radlat1 = this.degreesToRad(point1.latitude);
        /** @type {?} */
        const radlat2 = this.degreesToRad(point2.latitude);
        /** @type {?} */
        const theta = point1.longitude - point2.longitude;
        /** @type {?} */
        const radtheta = this.degreesToRad(theta);
        /** @type {?} */
        let dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
        if (dist > 1) {
            dist = 1;
        }
        dist = Math.acos(dist);
        dist = dist * 180 / Math.PI;
        dist = dist * 60 * 1.1515;
        // KM
        return dist * 1.609344;
    }
    /**
     * Converte os graus decimais em radianos
     * degrees Graus decimais
     * @param {?} degrees
     * @return {?}
     */
    degreesToRad(degrees) {
        return Math.PI * degrees / 180;
    }
    /**
     * Valida os valores do ponto recebido e retorna o resultado
     * point Ponto contendo latitude e longitude
     * @param {?} point
     * @return {?}
     */
    validGeoPositionPoint(point) {
        if (!point) {
            return false;
        }
        if (isNaN(point.latitude) || isNaN(point.longitude)) {
            return false;
        }
        if (point.latitude === null || point.longitude === null) {
            return false;
        }
        if (point.latitude > 90 || point.latitude < -90) {
            return false;
        }
        if (point.longitude > 180 || point.longitude < -180) {
            return false;
        }
        return true;
    }
}
GeolocationUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
GeolocationUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: Geolocation },
    { type: GlobalService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de geolocalização
 */
class GeolocationUtilModule {
}
GeolocationUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    Geolocation,
                    GeolocationUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de requisição http
 * nativa do dispositivo com métodos que facilitam a utilização do plugin
 */
class HttpOauth2UtilProvider {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * platform Serviço relacionado a plataforma
     * file Serviço do plugin nativo de arquivo
     * events Serviço de eventos do Ionic
     * @param {?} platform
     * @param {?} auth
     * @param {?} http
     */
    constructor(platform, auth, http) {
        this.auth = auth;
        this.http = http;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por realizar o upload de um arquivo
     * url URL do Endpoint de Upload
     * filePath URI do arquivo no dispositivo
     * name Nome do arquivo
     * body Informações no corpo da mensagem
     * headers Informações de cabeçalho da mensagem
     * {IHttpUtilResult} Resultado da requisição
     * @template T
     * @param {?} url
     * @param {?} filePath
     * @param {?=} name
     * @param {?=} body
     * @param {?=} headers
     * @return {?}
     */
    uploadFile(url, filePath, name, body, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                throw new CriticalError('Platform is not cordova (uploadFile)');
            }
            url = url.replace('{tenant}', this.auth.shortNameTenant || '');
            // Injeção da autorização Oauth2
            /** @type {?} */
            const user = yield this.auth.getUser();
            /** @type {?} */
            let postHeaders;
            if (headers) {
                postHeaders = Object.assign({}, headers, {
                    Authorization: user ? `${user.token_type} ${user.access_token}` : ''
                });
            }
            else {
                postHeaders = {
                    Authorization: user ? `${user.token_type} ${user.access_token}` : ''
                };
            }
            /** @type {?} */
            let postBody;
            if (body) {
                postBody = body;
            }
            else {
                postBody = {};
            }
            if (name === undefined || name === '') {
                name = 'file';
            }
            /** @type {?} */
            const result = {
                status: 0,
                data: undefined,
                error: undefined,
                sourceResponse: undefined
            };
            try {
                /** @type {?} */
                const response = yield this.http.uploadFile(url, postBody, postHeaders, filePath, name);
                result.sourceResponse = response;
                if (response) {
                    if (response.hasOwnProperty('status')) {
                        result.status = response.status;
                    }
                    if (response.hasOwnProperty('data')) {
                        try {
                            result.data = (/** @type {?} */ (JSON.parse(response.data)));
                        }
                        catch (error) {
                            result.error = error;
                        }
                    }
                }
            }
            catch (error) {
                result.error = error;
            }
            return result;
        });
    }
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * mode Modo de verificação do certificado SSL
     * @param {?} mode
     * @return {?}
     */
    setSSLCertMode(mode) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.http.setSSLCertMode(mode);
        });
    }
}
HttpOauth2UtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
HttpOauth2UtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: AuthBaseService },
    { type: HTTP }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de requisição http
 */
class HttpOauth2UtilModule {
}
HttpOauth2UtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    HTTP,
                    HttpOauth2UtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
class ImagePickerUtilEvents {
}
// FIXME - Nomes de eventos mais específicos, para evitar possíveis conflitos
/**
 * Evento publicado pelo utilitário quando uma ou mais imagens são
 * selecionadas e geram uma cópia no cache do aplicativo
 */
ImagePickerUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de seleção
 * de imagens do dispositivo com métodos que facilitam a solicitação de permissão,
 * apresentação e seleção de imagens
 */
class ImagePickerUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * imagePicker Serviço do plugin nativo de galeria
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} imagePicker
     * @param {?} global
     * @param {?} events
     */
    constructor(platform, imagePicker, global, events) {
        this.imagePicker = imagePicker;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        /**
         * @ignore
         * Opções padrões para a galeria
         */
        this.defaultOptions = {
            outputType: OutputType.FILE_URL,
            quality: 60
        };
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * opt Customização das opções enviadas ao seletor de images
     * Lista de imagens escolhidas
     * @param {?=} opt
     * @return {?}
     */
    chooseImage(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                /** @type {?} */
                const msg = new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                });
                this.global.msg.showMessage(msg);
                throw new CriticalError('Platform is not cordova');
            }
            /** @type {?} */
            let images = [];
            try {
                /** @type {?} */
                const options = Object.assign({}, this.defaultOptions);
                if (opt) {
                    for (const prop of Object.keys(opt)) {
                        options[prop] = opt[prop];
                    }
                }
                /** @type {?} */
                const hasPermission = yield this.requestReadPermission();
                if (hasPermission) {
                    images = yield this.imagePicker.getPictures(options);
                    this.events.publish(ImagePickerUtilEvents.CACHE_FILE_CREATED, ...images);
                }
                else {
                    /** @type {?} */
                    const msg = new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    });
                    this.global.msg.showMessage(msg);
                    throw new CriticalError('Permission denied');
                }
            }
            catch (error) {
                throw new CriticalError('Error in get pictures', error);
            }
            return images;
        });
    }
    /**
     * Método responsável por requisitar permissão para acesso a galeria
     * Indicação se possui ou não permissão
     * @return {?}
     */
    requestReadPermission() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let hasPermission = false;
            try {
                /** @type {?} */
                const permission = yield this.imagePicker.hasReadPermission();
                if (!permission) {
                    yield this.imagePicker.requestReadPermission();
                    /** @type {?} */
                    const permissionAfterRequest = yield this.imagePicker.hasReadPermission();
                    hasPermission = permissionAfterRequest;
                }
                else {
                    hasPermission = true;
                }
            }
            catch (error) {
                hasPermission = false;
            }
            return hasPermission;
        });
    }
}
ImagePickerUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ImagePickerUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: ImagePicker },
    { type: GlobalService },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de galeria
 */
class ImagePickerUtilModule {
}
ImagePickerUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    ImagePicker,
                    ImagePickerUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de leitura de QR Code
 * com métodos que facilitam a solicitação de permissão e leitura do código
 */
class QrScannerUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * qrScanner Serviço do plugin nativo de leitura do QR-Code
     * global Serviço global da aplicação
     * @param {?} qrScanner
     * @param {?} global
     */
    constructor(qrScanner, global) {
        this.qrScanner = qrScanner;
        this.global = global;
        /**
         * Propriedade que possui informações sobre o QR Code Scanner após ele estar
         * preparado, indicando assim se é possível ligar o flash ou trocar de camera,
         * por exemplo.
         */
        this.qrScannerLastStatus = new Subject();
    }
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * Indicação da preparação
     * @return {?}
     */
    prepare() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            this.qrScanner.prepare()
                .then((/**
             * @param {?} status
             * @return {?}
             */
            (status) => {
                this.qrScannerLastStatus.next(status);
                if (status.authorized) {
                    resolve(true);
                }
                else {
                    this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                    resolve(false);
                }
            }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                if (error && error.hasOwnProperty('name') && error.name === 'CAMERA_ACCESS_DENIED') {
                    this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                }
                else {
                    this.global.msg.showMessage(new BaseMessage({
                        literal: 'recursoNaoEstaDisponivelMsg'
                    }));
                }
                resolve(false);
            }));
        }));
    }
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * String com o conteúdo lido
     * @return {?}
     */
    show() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (!this.qrScannerLastStatus) {
                reject(undefined);
            }
            this.scanSub = this.qrScanner.scan().subscribe((/**
             * @param {?} result
             * @return {?}
             */
            (result) => {
                this.qrScanner.hide();
                this.scanSub.unsubscribe(); // stop scanning
                resolve(result);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                reject(error);
            }));
            this.qrScanner.show()
                .then((/**
             * @return {?}
             */
            () => { }))
                .catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => reject(error)));
        }));
    }
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     * @return {?}
     */
    hide() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status && status.showing) {
                this.qrScanner.hide().then((/**
                 * @param {?} statusAfter
                 * @return {?}
                 */
                (statusAfter) => {
                    this.qrScannerLastStatus.next(statusAfter);
                }));
            }
            if (status && status.scanning) {
                this.scanSub.unsubscribe();
            }
        }));
    }
    /**
     * Método responsável por ligar ou desligar o flash
     * @return {?}
     */
    flash() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status && status.scanning && status.canEnableLight) {
                if (status.lightEnabled) {
                    this.qrScanner.disableLight()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
                else {
                    this.qrScanner.enableLight()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
            }
        })).catch((/**
         * @return {?}
         */
        () => { }));
    }
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     * @return {?}
     */
    changeCamera() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status && status.scanning && status.canChangeCamera) {
                if (status.currentCamera === 0) { // Back camera
                    this.qrScanner.useFrontCamera()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
                else {
                    this.qrScanner.useBackCamera()
                        .then((/**
                     * @param {?} statusAfter
                     * @return {?}
                     */
                    (statusAfter) => {
                        this.qrScannerLastStatus.next(statusAfter);
                    }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
            }
        })).catch((/**
         * @return {?}
         */
        () => { }));
    }
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     * @return {?}
     */
    destroy() {
        this.qrScanner.getStatus().then((/**
         * @param {?} status
         * @return {?}
         */
        (status) => {
            if (status) {
                if (status.lightEnabled) {
                    this.qrScanner.disableLight()
                        .then((/**
                     * @return {?}
                     */
                    () => { }))
                        .catch((/**
                     * @return {?}
                     */
                    () => { }));
                }
                this.qrScanner.destroy()
                    .then((/**
                 * @return {?}
                 */
                () => { }))
                    .catch((/**
                 * @return {?}
                 */
                () => { }));
            }
        }));
    }
}
QrScannerUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
QrScannerUtilProvider.ctorParameters = () => [
    { type: QRScanner },
    { type: GlobalService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de leitura de QR-Code
 */
class QRScannerUtilModule {
}
QRScannerUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    QRScanner,
                    QrScannerUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de reconhecimento de voz
 * com métodos que facilitam a solicitação de permissão e reconhecimento
 */
class SpeechRecognitionUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * speechRecognition Serviço do plugin nativo de reconhecimento de voz
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} speechRecognition
     * @param {?} global
     */
    constructor(platform, speechRecognition, global) {
        this.speechRecognition = speechRecognition;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    // FIXME - Em relação a implementação no IOS, provavelmente será necessário implementar o stopListening()
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     * @return {?}
     */
    listening() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            this.verifyPermission().then((/**
             * @param {?} hasPermission
             * @return {?}
             */
            (hasPermission) => {
                if (hasPermission === false) {
                    this.showToast('permissaoNaoConcedidaParaORecursoMsg');
                    resolve([]);
                }
                else {
                    this.speechRecognition.isRecognitionAvailable()
                        .then((/**
                     * @param {?} available
                     * @return {?}
                     */
                    (available) => {
                        if (available === false) {
                            this.showToast('recursoNaoEstaDisponivelMsg');
                            resolve([]);
                            return;
                        }
                        this.speechRecognition.startListening(this.options)
                            .subscribe((/**
                         * @param {?} matches
                         * @return {?}
                         */
                        (matches) => resolve(matches)), (/**
                         * @param {?} onerror
                         * @return {?}
                         */
                        (onerror) => {
                            if (onerror && onerror instanceof Error && onerror.message !== '0') {
                                this.showToast('erroAoUtilizarRecursoMsg');
                            }
                            reject(onerror);
                        }));
                    })).catch((/**
                     * @param {?} error
                     * @return {?}
                     */
                    (error) => {
                        this.showToast('recursoNaoEstaDisponivelMsg');
                        reject(error);
                    }));
                }
            })).catch((/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                reject(error);
            }));
        }));
    }
    /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * Indicação se há ou não permissão
     * @private
     * @return {?}
     */
    verifyPermission() {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (this.isCordova === false) {
                this.showToast('recursoNaoEstaDisponivelMsg');
                reject('Cordova is not available.');
                return;
            }
            this.speechRecognition.hasPermission().then((/**
             * @param {?} hasPermission
             * @return {?}
             */
            (hasPermission) => {
                if (hasPermission) {
                    resolve(hasPermission);
                }
                else {
                    this.requestPermission()
                        .then((/**
                     * @param {?} requestPermissionSuccessful
                     * @return {?}
                     */
                    (requestPermissionSuccessful) => resolve(requestPermissionSuccessful)));
                }
            })).catch((/**
             * @return {?}
             */
            () => resolve(false)));
        }));
    }
    /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * Indicação se a permissão foi ou não concedida
     * @private
     * @return {?}
     */
    requestPermission() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            this.speechRecognition.requestPermission()
                .then((/**
             * @return {?}
             */
            () => resolve(true)), (/**
             * @return {?}
             */
            () => resolve(false))).catch((/**
             * @return {?}
             */
            () => resolve(false)));
        }));
    }
    /**
     * @ignore
     * Método para apresentar mensagens
     * literalMessage Literal da mensagem
     * @private
     * @param {?} literalMessage
     * @return {?}
     */
    showToast(literalMessage) {
        this.global.msg.showMessage(new BaseMessage({
            literal: literalMessage
        }));
    }
}
SpeechRecognitionUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SpeechRecognitionUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: SpeechRecognition },
    { type: GlobalService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de reconhecimento de voz
 */
class SpeechRecognitionUtilModule {
}
SpeechRecognitionUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    SpeechRecognition,
                    SpeechRecognitionUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo gerenciamento de variável em memória/storage do dispositivo
 * Ao utilizar os métodos da classe para atualizar o valor em memória, por padrão,
 * o valor é refletido no storage
 * @template T
 */
class StorageUtil {
    /**
     * Construtor da classe
     * storage Recebe o Storage via Angular Dependency Injection
     * @param {?} storage
     */
    constructor(storage) {
        this.storage = storage;
        /**
         * Campo id para facilitar buscas
         */
        this.fieldId = '';
        /**
         * Conteúdo a ser refletido
         */
        this.storageData = {};
        /**
         * Indica que a classe já foi inicializada corretamente
         */
        this.initialized = false;
        /**
         * Se uma operação que trava o registro está em andamento
         */
        this.lock = false;
    }
    /**
     * @return {?}
     */
    get id() {
        return this.fieldId;
    }
    /**
     * @return {?}
     */
    get exclusive() {
        return this.exclusiveStorage;
    }
    /**
     * @return {?}
     */
    get suffixFn() {
        if (!this.suffixFunction) {
            return (/**
             * @return {?}
             */
            () => '');
        }
        else {
            return this.suffixFunction;
        }
    }
    /**
     * @return {?}
     */
    get initFn() {
        return this.initFunc;
    }
    /**
     * @return {?}
     */
    get name() {
        return this.storageName;
    }
    /**
     * @return {?}
     */
    get data() {
        if (!this.isInit) {
            throw new Error(`Storage: ${this.name} está sendo lido sem ser inicializado.`);
        }
        return this.getCurrentSuffixData();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set data(value) {
        if (!this.isInit) {
            throw new Error(`Storage: ${this.name} está sendo definido sem ser inicializado.`);
        }
        /** @type {?} */
        const suffix = this.currentSuffix;
        this.storageData[suffix] = value;
    }
    /**
     * Sufixo corrente quando storage exclusivo
     * @return {?}
     */
    get currentSuffix() {
        /** @type {?} */
        let suffix = this.suffixFn();
        if (!suffix || suffix === '') {
            suffix = StorageUtil.SUFFIX_DEFAULT;
        }
        return suffix;
    }
    /**
     * @return {?}
     */
    get isInit() {
        return this.initialized;
    }
    /**
     * @return {?}
     */
    get isLock() {
        return this.lock;
    }
    /**
     * Configura e realiza a leitura de dados do storage
     * { name:string, id? :string, exclusive?: boolean, suffixFn?: () => void, lazy?: boolean, initFn?: () => void}
     * @param {?=} __0
     * @return {?}
     */
    config({ name, id, exclusive, suffixFn, lazy, initFn, } = {
        name: '',
        id: '',
        exclusive: false,
        suffixFn: (/**
         * @return {?}
         */
        () => ''),
        lazy: false,
        initFn: (/**
         * @return {?}
         */
        () => undefined),
    }) {
        if (!name || name === '') {
            throw new Error('Storage necessita de um nome (name)');
        }
        if (exclusive && !suffixFn) {
            throw new Error('Storage exclusivo necessita de uma função para retornar o sufixo (suffixFn)');
        }
        this.storageName = name;
        this.fieldId = id;
        this.exclusiveStorage = exclusive;
        this.suffixFunction = suffixFn;
        this.initFunc = initFn;
        if (lazy) {
            return Promise.resolve(undefined);
        }
        else {
            return this.read();
        }
    }
    /**
     * Atualiza o storage para refletir o valor contido em memória
     * @return {?}
     */
    updateStorage() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isInit) {
                throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
            }
            yield UtilsFn.lock((/**
             * @return {?}
             */
            () => this.lock));
            this.lock = true;
            return yield this.storage
                .set(this.name, this.storageData)
                .then((/**
             * @return {?}
             */
            () => {
                this.lock = false;
                return this.data;
            }))
                .catch((/**
             * @param {?} err
             * @return {?}
             */
            err => {
                this.lock = false;
                console.error(err);
                return undefined;
            }));
        });
    }
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * search
     * fields
     * \@example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     * @param {?} search
     * @param {...?} fields
     * @return {?}
     */
    find(search, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const array = this.data;
            array.find((/**
             * @param {?} element
             * @return {?}
             */
            element => {
                /** @type {?} */
                const ret = this.recursiveField(fields, element) === search;
                if (ret) {
                    fields.pop();
                    if (fields.length > 0) {
                        return this.recursiveField(fields, element);
                    }
                    else {
                        return element;
                    }
                }
                return ret;
            }));
            throw new Error('Item não encontrado no storage: ' + search);
        });
    }
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * \@example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * newData
     * save
     * fields
     * @param {?} newData
     * @param {?=} save
     * @param {...?} fields
     * @return {?}
     */
    set(newData, save = true, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isInit) {
                throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
            }
            if (fields.length === 0) {
                this.data = newData;
            }
            else {
                /** @type {?} */
                const field = fields.pop();
                /** @type {?} */
                const data = this.recursiveField(fields, this.data);
                data[field] = newData;
            }
            if (save) {
                return yield this.updateStorage();
            }
            else {
                return this.data;
            }
        });
    }
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * search Busca em campo que representa uma chave primária do objeto
     * @param {?} search
     * @return {?}
     */
    get(search) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.read();
            if (this.id && search) {
                try {
                    /** @type {?} */
                    const array = this.data;
                    return array.find((/**
                     * @param {?} element
                     * @return {?}
                     */
                    element => {
                        return element[this.id] === search;
                    }));
                }
                catch (error) {
                    throw error;
                }
            }
            else {
                return this.data;
            }
        });
    }
    /**
     * Faz a leitura dos dados que estão no storage
     * @return {?}
     */
    read() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.name) {
                throw new Error('Necessário chamar o método config');
            }
            yield UtilsFn.lock((/**
             * @return {?}
             */
            () => this.lock));
            /** @type {?} */
            const suffix = this.currentSuffix;
            if (this.storageData.hasOwnProperty(suffix)) {
                return this.data;
            }
            else if (this.isInit) {
                this.storageData[suffix] = this.init(null);
            }
            else {
                return yield this.storage
                    .get(this.name)
                    .then((/**
                 * @param {?} value
                 * @return {?}
                 */
                value => {
                    return this.updateData(value);
                }))
                    .catch((/**
                 * @param {?} err
                 * @return {?}
                 */
                err => {
                    console.error(err);
                    return undefined;
                }));
            }
        });
    }
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * search
     * @param {?} search
     * @return {?}
     */
    delete(search) {
        if (!this.isInit) {
            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
        }
        if (this.id && search) {
            return new Promise((/**
             * @param {?} resolve
             * @param {?} reject
             * @return {?}
             */
            (resolve, reject) => {
                /** @type {?} */
                const array = this.data;
                /** @type {?} */
                const index = array.findIndex((/**
                 * @param {?} element
                 * @return {?}
                 */
                element => {
                    return element[this.id] === search;
                }));
                if (index > -1) {
                    array.splice(index, 1);
                    this.set(array).then((/**
                     * @param {?} v
                     * @return {?}
                     */
                    v => {
                        resolve(this.data);
                    }));
                }
                else {
                    reject(`Registro ${search} não encontrado.`);
                }
            }));
        }
    }
    /**
     * Atualiza o valor em memória lida do storage
     * value
     * @private
     * @param {?} value
     * @return {?}
     */
    updateData(value) {
        if (value === undefined || value === null || typeof value !== 'object') {
            this.storageData = {};
        }
        else {
            this.storageData = value;
        }
        this.initialized = true;
        return this.data;
    }
    /**
     * Inicializa o armazenamento com base na função que o usuário informou
     * @private
     * @param {?} data
     * @return {?}
     */
    init(data) {
        if ((data === undefined || data === null) && this.initFn) {
            data = this.initFn();
        }
        return data;
    }
    /**
     * Função recursíva para encontrar subníveis automáticamente
     * x array de campos recursivos
     * data a informação
     * level nível de iteração
     * @private
     * @param {?} fields
     * @param {?} data
     * @param {?=} level
     * @return {?}
     */
    recursiveField(fields, data, level = 0) {
        if (level < fields.length - 1) {
            return this.recursiveField(fields, data[fields[level]], ++level);
        }
        return data[fields[level]];
    }
    /**
     * Recupera os dados do sufíxo atual
     * @private
     * @return {?}
     */
    getCurrentSuffixData() {
        /** @type {?} */
        const suffix = this.currentSuffix;
        if (this.storageData.hasOwnProperty(suffix) === false) {
            this.storageData[suffix] = this.init(null);
        }
        return this.storageData[suffix];
    }
}
/**
 * Sufixo padrão
 */
StorageUtil.SUFFIX_DEFAULT = '__stDefault0__';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Gerenciador de conexão de banco de dados sqlite
 * @abstract
 */
class SqLiteUtilBaseProvider {
    /**
     * @param {?} sqlLite
     * @param {?} platform
     */
    constructor(sqlLite, platform) {
        this.sqlLite = sqlLite;
        this.platform = platform;
    }
    /**
     * Método para auxiliar a criar uma conexão com o banco sqlite
     * name Nome do banco
     * location Localização do banco
     * DbConnectionBaseProvider
     * @protected
     * @param {?} name
     * @param {?} location
     * @return {?}
     */
    createConn(name, location) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.platform.ready();
            return this.sqlLite.create({ name, location });
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Gerenciador de conexão de banco de dados sqlite
 */
class SqLiteUtilProvider extends SqLiteUtilBaseProvider {
    /**
     * @param {?} sqlLite
     * @param {?} platform
     */
    constructor(sqlLite, platform) {
        super(sqlLite, platform);
        this.connections = new Array();
        this.creating = new Array();
    }
    /**
     * Recupera uma conexão de banco de dados
     *
     * name o banco de dados desejado
     * {string} [location='default'] Configuração adicional de localização no dispositivo
     * @param {?} name
     * @param {?=} location
     * @return {?} Um objeto de conexão com o banco
     */
    getSqLiteConn(name, location = 'default') {
        return __awaiter(this, void 0, void 0, function* () {
            yield UtilsFn.lock((/**
             * @return {?}
             */
            () => this.creating.find((/**
             * @param {?} x
             * @return {?}
             */
            (x) => x.name === name && x.location === location))));
            /** @type {?} */
            const element = this.connections.find((/**
             * @param {?} x
             * @return {?}
             */
            (x) => x.name === name && x.location === location));
            /** @type {?} */
            let sqlConn;
            if (!element) {
                this.creating.push({ name, location, conn: undefined });
                sqlConn = yield this.createConn(name, location);
                this.connections.push({ name, location, conn: sqlConn });
                /** @type {?} */
                const idx = this.creating.findIndex((/**
                 * @param {?} x
                 * @return {?}
                 */
                (x) => x.name === name && x.location === location));
                if (idx !== -1) {
                    this.creating.splice(idx, 1);
                }
            }
            else {
                sqlConn = element.conn;
            }
            return sqlConn;
        });
    }
}
SqLiteUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SqLiteUtilProvider.ctorParameters = () => [
    { type: SQLite },
    { type: Platform }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que controla as conexões com banco sqlite
 * e provê conexão com o serviço SqLite
 * Usar SqLiteUtilModule.forRoot() ao importar
 * @export
 * SqLiteUtilModule
 */
class SqLiteUtilModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: SqLiteUtilModule,
            providers: [
                SQLite,
                {
                    provide: SqLiteUtilBaseProvider,
                    useClass: SqLiteUtilProvider
                }
            ]
        };
    }
}
SqLiteUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: []
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe abstrata que todo recurso de sincronização deve implementar,
 * independente do sentido
 * Para registrar o serviço em que a sincronização automática reconheça:
 * export const entidadeToken = new InjectionToken<entidadeProvider>('');
 * export class entidadeProvider extends SyncHandler{}
 * {
 * provide: entidadeToken,
 * useClass: entidadeProvider
 * }
 * @abstract
 */
class SyncHandler {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe que contém todos os eventos disponíbilizados pelo serviço gerenciador de
 * sincronismo
 */
class SyncEvents {
}
/**
 * Evento que o sincronizador dispara ao terminar um sincronismo.
 * Deve-se filtrar pelo nome da entidade
 */
SyncEvents.GOT_ENTITY_SYNC_DATA = 'Sync:GotEntitySyncData';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const SYNC_CONFIG = new InjectionToken('SYNC_CONFIG: ISyncConfig token de configuração da sincronização');
/**
 * Serviço provedor de sincronismo utilizando serviços externos
 */
class SyncProvider {
    /**
     * Construtor da classe
     * platform Platform do ionic
     * authService Serviço de autenticação
     * storage Serviço de armazenamento do ionic
     * config Token de configuração do serviço
     * network Serviço de rede do ionic
     * events Serviço de eventos do ionic
     * injector Injetor de dependências do angular
     * @param {?} platform
     * @param {?} storage
     * @param {?} config
     * @param {?} network
     * @param {?} events
     * @param {?} injector
     */
    constructor(platform, storage, config, network, events, injector) {
        this.storage = storage;
        this.config = config;
        this.network = network;
        this.events = events;
        this.injector = injector;
        /**
         * @ignore
         */
        this.syncing = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => __awaiter(this, void 0, void 0, function* () {
            if (platform.is('cordova')) {
                this.syncStorage = new StorageUtil(this.storage);
                yield this.syncStorage.config({
                    name: 'SyncEntitiesState',
                    exclusive: true,
                    lazy: true,
                    suffixFn: (/**
                     * @return {?}
                     */
                    () => this.storageName),
                    initFn: (/**
                     * @return {?}
                     */
                    () => {
                        /** @type {?} */
                        const init = [];
                        config.entities.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => init.push(this.newState(e))));
                        return init;
                    })
                });
                this.setDefaults();
                this.subscribeEvents();
            }
        })));
    }
    /**
     * Se a sincronização está acontecendo
     * @return {?}
     */
    get isSyncing() {
        return this.syncing;
    }
    /**
     * @param {?} storageNameFunction
     * @return {?}
     */
    set storageNameFunction(storageNameFunction) {
        this.storageNameFn = storageNameFunction;
    }
    /**
     * Atributo que retorna o nome do storage
     * @return {?}
     */
    get storageName() {
        if (this.storageNameFn) {
            return this.storageNameFn();
        }
        else {
            return 'sync';
        }
    }
    /**
     * Retorna se o sincronismo está ativo
     * @return {?}
     */
    get autoWatchEnabled() {
        if (this.currentNetworkType === 'none') {
            return false;
        }
        if (!this.config.autoWatch) {
            return false;
        }
        if (this.config.autoWatch.enabledOnNetwork === 'any' || this.config.autoWatch.enabledOnNetwork === this.currentNetworkType) {
            return true;
        }
        else {
            /** @type {?} */
            const array = (/** @type {?} */ (this.config.autoWatch.enabledOnNetwork));
            if (array.findIndex((/**
             * @param {?} x
             * @return {?}
             */
            (x) => x === 'any' || x === this.currentNetworkType)) !== -1) {
                return true;
            }
        }
        return false;
    }
    /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     * @return {?}
     */
    syncAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.syncing === false) {
                this.syncing = true;
                try {
                    yield this.syncStorage.read();
                    this.syncAllByOrder(this.config.entities);
                }
                catch (error) {
                    console.error('Sync autowatch stopped. Reason', error);
                    this.syncing = false;
                    this.stopAutoWatch();
                }
            }
        });
    }
    /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * entities Lista com todas as entidades
     * order Ordem da execução
     * @param {?} entities
     * @param {?=} order
     * @return {?}
     */
    syncAllByOrder(entities, order = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const entitiesByOrder = [];
            entities.forEach((/**
             * @param {?} e
             * @return {?}
             */
            (e) => {
                if ((order === 0 && (e.order === undefined || e.order === 0)) ||
                    (order !== 0 && (e.order === order))) {
                    entitiesByOrder.push(e);
                }
            }));
            if (entitiesByOrder.length > 0) {
                /** @type {?} */
                const result = yield this.syncAllEntities(entitiesByOrder);
                // Aborta demais sincronizações
                if (!result) {
                    return;
                }
            }
            if (entities.some((/**
             * @param {?} e
             * @return {?}
             */
            (e) => e.order > order))) {
                this.syncAllByOrder(entities, order + 1);
            }
            else {
                this.syncing = false;
            }
        });
    }
    /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * entities Lista com as entidades que deverão ser sincronizadas
     * Indicação se ocorreu com sucesso
     * @param {?} entities
     * @return {?}
     */
    syncAllEntities(entities) {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        (resolve) => {
            /** @type {?} */
            const requests = [];
            entities.forEach((/**
             * @param {?} e
             * @return {?}
             */
            (e) => {
                /** @type {?} */
                const provider = (/** @type {?} */ (this.injector.get(e.providerToken)));
                /** @type {?} */
                const state = this.getState(e);
                state.isSyncing = true;
                requests.push(provider.getSyncData(state));
            }));
            forkJoin(requests).subscribe((/**
             * @param {?} x
             * @return {?}
             */
            (x) => {
                if (x) {
                    x.forEach((/**
                     * @param {?} e
                     * @return {?}
                     */
                    (e) => {
                        if (e) {
                            this.syncStorage.data[this.getStateIdx(e.state.entity)] = e.state;
                        }
                    }));
                    this.syncStorage.updateStorage()
                        .then((/**
                     * @return {?}
                     */
                    () => {
                        x.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => {
                            this.events.publish(SyncEvents.GOT_ENTITY_SYNC_DATA, e);
                        }));
                        resolve(true);
                    }));
                }
                else {
                    this.syncing = false;
                    resolve(false);
                }
            }), (/**
             * @return {?}
             */
            () => {
                this.syncing = false;
                resolve(false);
            }));
        }));
    }
    /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     * @param {?} now
     * @return {?}
     */
    startAutoWatch(now) {
        this.watchNetwork();
        this.stopAutoWatch();
        if (now) {
            this.nowToken = setTimeout((/**
             * @return {?}
             */
            () => {
                if (this.autoWatchEnabled) {
                    this.syncAll();
                }
            }));
            this.intervalToken = setInterval((/**
             * @return {?}
             */
            () => {
                if (this.autoWatchEnabled) {
                    this.syncAll();
                }
            }), this.getNextCall());
        }
        else {
            this.scheduleToken = of({}).pipe(delay(this.getNextCall())).subscribe((/**
             * @return {?}
             */
            () => {
                this.intervalToken = setInterval((/**
                 * @return {?}
                 */
                () => {
                    if (this.autoWatchEnabled) {
                        this.syncAll();
                    }
                }), this.getNextCall());
            }));
        }
    }
    /**
     * Cancela a sincronização automática.
     * @return {?}
     */
    stopAutoWatch() {
        // TODO: Implementar uma espera da sincronização em andamento terminar antes de encerrar os timers ativos.
        if (this.nowToken) {
            clearInterval(this.nowToken);
            this.nowToken = undefined;
        }
        if (this.scheduleToken) {
            this.scheduleToken.unsubscribe();
            this.scheduleToken = undefined;
        }
        if (this.intervalToken) {
            clearInterval(this.intervalToken);
            this.intervalToken = undefined;
        }
    }
    /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * types
     * @param {...?} types
     * @return {?}
     */
    changeEnabledAutoWatchNetWorks(...types) {
        this.config.autoWatch.enabledOnNetwork = types;
        this.setDefaults();
    }
    /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * newConfiguration Novas configurações
     * @param {?=} newConfiguration
     * @return {?}
     */
    changeSyncWatch(newConfiguration) {
        this.config.autoWatch = newConfiguration;
        if (this.config.autoWatch) {
            this.startAutoWatch(true);
        }
    }
    /**
     * Recupera o estado de sincronização da entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    getState(entity) {
        /** @type {?} */
        let e = this.syncStorage.data.find((/**
         * @param {?} x
         * @return {?}
         */
        (x) => x.entity.name === entity.name));
        if (e === undefined) {
            /** @type {?} */
            const newEntity = this.newState(entity);
            this.syncStorage.data.push(newEntity);
            e = newEntity;
        }
        return e;
    }
    /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    getStateIdx(entity) {
        /** @type {?} */
        let idx = this.syncStorage.data.findIndex((/**
         * @param {?} x
         * @return {?}
         */
        (x) => x.entity.name === entity.name));
        if (idx === -1) {
            /** @type {?} */
            const newEntity = this.newState(entity);
            idx = this.syncStorage.data.push(newEntity) - 1;
        }
        return idx;
    }
    /**
     * Cria um novo estado de sincronização para a entidade
     * entity a entidade de sincronização
     * @private
     * @param {?} entity
     * @return {?}
     */
    newState(entity) {
        return { entity, entityVersion: undefined, isSyncing: false };
    }
    /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     * @private
     * @return {?}
     */
    getNextCall() {
        if (this.config.autoWatch) {
            /** @type {?} */
            let times = this.config.autoWatch.checkQuantity;
            if (times === undefined || times <= 0 || isNaN(times)) {
                times = 1;
            }
            /** @type {?} */
            const type = this.config.autoWatch.checkType;
            if (type === 'days') {
                return SyncProvider.daysMs * times;
            }
            else if (type === 'hours') {
                return SyncProvider.hoursMs * times;
            }
            else if (type === 'minutes') {
                return SyncProvider.minutesMs * times;
            }
            else if (type === 'seconds') {
                return SyncProvider.secondsMs * times;
            }
            else { // Defaults for new types
                return SyncProvider.minutesMs * times;
            }
        }
        else {
            return SyncProvider.minutesMs;
        }
    }
    /**
     * @ignore
     * Configura os valores padrão, quando não informados
     * @private
     * @return {?}
     */
    setDefaults() {
        if (this.config.autoWatch) {
            if (this.config.autoWatch.enabledOnNetwork instanceof Array) {
                /** @type {?} */
                const array = (/** @type {?} */ (this.config.autoWatch.enabledOnNetwork));
                if (array.length === 0) {
                    this.config.autoWatch.enabledOnNetwork = 'any';
                }
            }
        }
    }
    /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     * @private
     * @return {?}
     */
    subscribeEvents() {
        if (this.config.autoWatch) {
            this.currentNetworkType = this.network.type;
            if (this.config.autoWatch.auto) {
                this.startAutoWatch(false);
            }
            this.watchNetwork();
        }
    }
    /**
     * Assina o evento de trocas de rede
     * @private
     * @return {?}
     */
    watchNetwork() {
        if (!this.onNetworkChange) {
            this.onNetworkChange = this.network.onchange().subscribe((/**
             * @return {?}
             */
            () => {
                setTimeout((/**
                 * @return {?}
                 */
                () => {
                    this.currentNetworkType = this.network.type;
                }), 3000);
            }));
        }
    }
}
/**
 * Constante convertendo 1 dia em milisegundos
 */
SyncProvider.daysMs = 86400000;
/**
 * Constante convertendo 1 hora em milisegundos
 */
SyncProvider.hoursMs = 3600000;
/**
 * Constante convertendo 1 minuto em milisegundos
 */
SyncProvider.minutesMs = 60000;
/**
 * Constante convertendo 1 segundo em milisegundos
 */
SyncProvider.secondsMs = 1000;
SyncProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
SyncProvider.ctorParameters = () => [
    { type: Platform },
    { type: Storage },
    { type: undefined, decorators: [{ type: Inject, args: [SYNC_CONFIG,] }] },
    { type: Network },
    { type: Events },
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo com as dependências para sincronização mobile
 */
class SyncModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: SyncModule,
            providers: [
                SyncProvider,
                Network
            ]
        };
    }
}
SyncModule.decorators = [
    { type: NgModule, args: [{},] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Utilitário com funções para sincronização de dados em banco sqlite
 */
class SyncSqLiteUtil {
    /**
     * Creates an instance of SyncDbUtils.
     * cn a conexão que será utilizada para salvar os dados
     * @param {?} cn
     */
    constructor(cn) {
        this.cn = cn;
    }
    /**
     * Cria um relacionamento 1-n, inserindo na entidade filha o id do pai
     *
     * items os registros pai
     * fatherTable o nome da tabela pi
     * @template T
     * @param {?} items
     * @param {?} fatherTable
     * @return {?} Os registros relacionados
     */
    updateChildrenRelation(items, fatherTable) {
        // FIXME: Fazer com que todos os filhos também tenha relacionamento com o pai mestre
        /** @type {?} */
        const childrenFound = [];
        /** @type {?} */
        const models = Object.keys(items[0]);
        for (const item of items) {
            for (const model of models) {
                if (UtilsFn.isArray(item[model])) {
                    /** @type {?} */
                    const children = (/** @type {?} */ (item[model]));
                    /** @type {?} */
                    const cf = childrenFound.find((/**
                     * @param {?} x
                     * @return {?}
                     */
                    (x) => x.table === model));
                    if (!cf) {
                        childrenFound.push({ table: model, items: children });
                    }
                    else {
                        cf.items = cf.items.concat(children);
                    }
                    for (const child of children) {
                        child[`${fatherTable}Id`] = item.id;
                    }
                }
            }
        }
        return childrenFound;
    }
    /**
     * Armazena os dados enviados, substituindo o registro anterior e criando tabelas para
     * entidades filhas(arrays)
     * data A requisição de sync com os dados
     * mainTable O nome da tabela principal
     * deleteBefore se deve sempre deletar antes de inserir o registro
     * currentTx a transação corrente, não deve ser passado sem entender o mecanismo interno
     * @template T
     * @param {?} data
     * @param {?} mainTable
     * @param {?=} deleteBefore
     * @param {?=} currentTx
     * @return {?}
     */
    storeData(data, mainTable, deleteBefore = true, currentTx) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!data.items || data.items.length === 0) {
                return;
            }
            /** @type {?} */
            const children = this.updateChildrenRelation(data.items, mainTable);
            /** @type {?} */
            const model = this.modelWithoutArrays(data.items[0]);
            if (!currentTx) {
                try {
                    yield this.cn.transaction((/**
                     * @param {?} tx
                     * @return {?}
                     */
                    (tx) => {
                        this.checkTableStateTx(tx, mainTable, model, (/**
                         * @param {?} txa
                         * @param {?} rs
                         * @return {?}
                         */
                        (txa, rs) => {
                            for (const item of data.items) {
                                this.upsert(txa, mainTable, model, item, deleteBefore);
                            }
                            for (const child of children) {
                                this.storeData({
                                    items: child.items
                                }, `${mainTable}_${child.table}`, deleteBefore, tx);
                            }
                        }));
                    }));
                }
                catch (error) {
                    console.log(error);
                }
            }
            else { // Continua na transação corrente
                this.checkTableStateTx(currentTx, mainTable, model, (/**
                 * @param {?} tx
                 * @param {?} rs
                 * @return {?}
                 */
                (tx, rs) => {
                    if (mainTable.search('_') !== -1 && data.items.length > 0) {
                        // Delete childs first
                        /** @type {?} */
                        const tables = mainTable.split('_');
                        tables.splice(tables.length - 1, 1);
                        /** @type {?} */
                        const fieldId = tables.join('_') + 'Id';
                        /** @type {?} */
                        const fathersId = [];
                        data.items.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => {
                            if (!fathersId.find((/**
                             * @param {?} x
                             * @return {?}
                             */
                            (x) => x === e[fieldId]))) {
                                fathersId.push(e[fieldId]);
                            }
                        }));
                        fathersId.forEach((/**
                         * @param {?} e
                         * @return {?}
                         */
                        (e) => {
                            console.log(`DELETE FROM ${mainTable} where ${fieldId} = ?`, [e]);
                            tx.executeSql(`DELETE FROM ${mainTable} where ${fieldId} = ?`, [e], (/**
                             * @param {?} txa
                             * @param {?} res
                             * @return {?}
                             */
                            (txa, res) => { }), (/**
                             * @param {?} error
                             * @return {?}
                             */
                            (error) => console.log(error)));
                        }));
                    }
                    for (const item of data.items) {
                        this.upsert(currentTx, mainTable, model, item, deleteBefore);
                    }
                    for (const child of children) {
                        this.storeData({
                            items: child.items
                        }, `${mainTable}_${child.table}`, deleteBefore, currentTx);
                    }
                }));
            }
        });
    }
    /**
     * Retorna se a tabela existe
     * table o nome da tabela que se deseja verificar se existe
     * @param {?} table
     * @return {?}
     */
    tableExists(table) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let rs;
            try {
                rs = yield this.cn.executeSql('SELECT name FROM sqlite_master WHERE type=? AND name=?', ['table', table]);
            }
            catch (error) {
                console.log(JSON.stringify(error, null, 2));
            }
            if (rs && rs.rows.length > 0 && rs.rows.item(0).name === table) {
                return true;
            }
            return false;
        });
    }
    /**
     * Cria a tabela desejada, se ela não existir
     * table o nome da tabela
     * fields os campos pertencentes a tabela
     * @param {?} table
     * @param {...?} fields
     * @return {?}
     */
    createTable(table, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.cn.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + fields.join(',') + ')');
            }
            catch (rs) {
                console.log('success', rs);
            }
        });
    }
    /**
     * Verifica se os campos já existem na tabela e os adiciona
     * table O nome da tabela
     * fields os campos que serão adicionados, se ainda não existirem na tabela
     * @param {?} table
     * @param {...?} fields
     * @return {?}
     */
    checkAndAddFields(table, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.cn.executeSql('PRAGMA table_info(' + table + ')');
            }
            catch (rs) {
                for (const field of fields) {
                    /** @type {?} */
                    const hasField = this.checkField(rs, field);
                    if (hasField === false) {
                        try {
                            yield this.cn.executeSql(`ALTER TABLE ${table} ADD ${field}`);
                        }
                        catch (rs) {
                            console.log('success', rs);
                        }
                    }
                }
            }
        });
    }
    /**
     * Retorna um array com os nomes dos campos da entidade, removendo os que são do tipo array
     * obj o objeto que se deseja avaliar
     * @private
     * @param {?} obj
     * @return {?}
     */
    modelWithoutArrays(obj) {
        /** @type {?} */
        const model = Object.keys(obj);
        for (let i = model.length - 1; i > -1; i--) {
            /** @type {?} */
            const field = model[i];
            if (UtilsFn.isArray(obj[field])) {
                model.splice(i, 1);
            }
        }
        return model;
    }
    /**
     * Executa o sql necessário para realizar o upsert de um registro, dentro de uma transação
     * tx a transação corrente
     * table o nome da tabela
     * model os campos envolvidos
     * item o item que se deseja inserir
     * deleteBefore se deve sempre excluir o registro antes de inserir
     * @private
     * @param {?} tx
     * @param {?} table
     * @param {?} model
     * @param {?} item
     * @param {?} deleteBefore
     * @return {?}
     */
    upsert(tx, table, model, item, deleteBefore) {
        if (deleteBefore) {
            if (table.search('_') === -1) { // Pula deleção de tabelas-filha
                console.log(`DELETE FROM ${table} where id = ?`, [item.id]);
                tx.executeSql(`DELETE FROM ${table} where id = ?`, [item.id], (/**
                 * @param {?} txa
                 * @param {?} rsa
                 * @return {?}
                 */
                (txa, rsa) => { }), (/**
                 * @param {?} error
                 * @return {?}
                 */
                (error) => console.log(error)));
            }
        }
        console.log(`INSERT INTO ${table} (${model.join(',')}) VALUES(${SqlUtilsFn.objectToSqlstring(item, model)})`);
        // tslint:disable-next-line:max-line-length
        tx.executeSql(`INSERT INTO ${table} (${model.join(',')}) VALUES(${SqlUtilsFn.objectToSqlstring(item, model)})`, undefined, (/**
         * @param {?} txa
         * @param {?} rsa
         * @return {?}
         */
        (txa, rsa) => { }), (/**
         * @param {?} error
         * @return {?}
         */
        (error) => console.log(error)));
    }
    /**
     * Na consulta de campos, verifica se o campo está presente na tabela
     * rs o recordset com os registros
     * field o nome do campo desejado
     * @private
     * @param {?} rs
     * @param {?} field
     * @return {?}
     */
    checkField(rs, field) {
        /** @type {?} */
        let hasField = false;
        for (let i = 0; i < rs.rows.length; i++) {
            if (rs.rows.item(i).name === field) {
                hasField = true;
                break;
            }
        }
        return hasField;
    }
    /**
     * Cria ou atualização a estrutura da tabela dentro de uma transação
     * tx a transação corrente
     * mainTable o nome da tabela
     * model os campos da tabela
     * afterTx o que fazer após a transação ocorrer com sucesso
     * @private
     * @param {?} tx
     * @param {?} mainTable
     * @param {?} model
     * @param {?} afterTx
     * @return {?}
     */
    checkTableStateTx(tx, mainTable, model, afterTx) {
        tx.executeSql('SELECT name FROM sqlite_master WHERE type=? AND name=?', ['table', mainTable], (/**
         * @param {?} txa
         * @param {?} rsa
         * @return {?}
         */
        (txa, rsa) => {
            if (rsa && rsa.rows.length > 0 && rsa.rows.item(0).name === mainTable) {
                txa.executeSql('PRAGMA table_info(' + mainTable + ')', undefined, (/**
                 * @param {?} txb
                 * @param {?} rsb
                 * @return {?}
                 */
                (txb, rsb) => {
                    /** @type {?} */
                    let qtd = 0;
                    /** @type {?} */
                    const fieldsToAdd = [];
                    for (const field of model) {
                        /** @type {?} */
                        const hasField = this.checkField(rsb, field);
                        if (hasField === false) {
                            qtd++;
                            fieldsToAdd.push(field);
                        }
                    }
                    if (fieldsToAdd.length === 0) {
                        afterTx(txb, rsb);
                    }
                    for (const field of fieldsToAdd) {
                        txb.executeSql(`ALTER TABLE ${mainTable} ADD ${field}`, undefined, (/**
                         * @param {?} txc
                         * @param {?} rsc
                         * @return {?}
                         */
                        (txc, rsc) => {
                            qtd--;
                            if (qtd === 0) {
                                afterTx(txc, rsc);
                            }
                        }));
                    }
                }));
            }
            else {
                txa.executeSql('CREATE TABLE IF NOT EXISTS ' + mainTable + ' (' + model.join(',') + ')', undefined, (/**
                 * @param {?} txb
                 * @param {?} rsb
                 * @return {?}
                 */
                (txb, rsb) => {
                    afterTx(txb, rsb);
                }));
            }
        }));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MultiTenantService {
    /**
     * Método construtor responsável por inicializar o serviço
     * @param {?} platform Serviço relacionado a plataforma
     * @param {?} storage Serviço relacionado ao storage
     * @param {?} events
     */
    constructor(platform, storage, events) {
        this.storage = storage;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        this.ready = new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            this.readyFn = resolve;
        }));
        platform.ready().then((/**
         * @return {?}
         */
        () => {
            this.isCordova = platform.is('cordova');
            if (this.isCordova) {
                this.tenantsStorage = new StorageUtil(this.storage);
                this.tenantsStorage
                    .config({
                    name: 'Tenants',
                    id: 'shortNameTenant',
                    exclusive: false,
                    lazy: false,
                    initFn: (/**
                     * @return {?}
                     */
                    () => []),
                })
                    .then((/**
                 * @return {?}
                 */
                () => { }));
                this.currentTenantStorage = new StorageUtil(this.storage);
                this.currentTenantStorage
                    .config({
                    name: 'CurrentTenant',
                    id: 'shortNameTenant',
                    exclusive: false,
                    lazy: true,
                    initFn: (/**
                     * @return {?}
                     */
                    () => undefined),
                })
                    .then((/**
                 * @return {?}
                 */
                () => { }));
                this.readyFn();
            }
        }));
    }
    /**
     * @return {?}
     */
    get tenants() {
        return this.tenantsStorage ? this.tenantsStorage.data : [];
    }
    /**
     * @return {?}
     */
    get isReady() {
        return this.ready;
    }
    /**
     * @return {?}
     */
    getCurrentTenant() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            /** @type {?} */
            const currentTenant = (/** @type {?} */ ((yield this.currentTenantStorage.read())));
            try {
                if (yield this.tenantsStorage.get(currentTenant.shortNameTenant)) {
                    return currentTenant;
                }
            }
            catch (error) {
                // console.error(error);
                this.currentTenantStorage.data = undefined;
                yield this.currentTenantStorage.updateStorage();
                return undefined;
            }
        });
    }
    /**
     * @param {?} currentTenant
     * @return {?}
     */
    setCurrentTenant(currentTenant) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            yield this.currentTenantStorage.read();
            if (!this.currentTenantStorage.data) {
                this.currentTenantStorage.data = currentTenant;
            }
            /** @type {?} */
            const tenantChanged = currentTenant.shortNameTenant !==
                this.currentTenantStorage.data.shortNameTenant;
            this.currentTenantStorage.data = currentTenant;
            yield this.currentTenantStorage.updateStorage();
            if (tenantChanged) {
                this.events.publish(MultiTenantEvents.CURRENT_TENANT_CHANGED, currentTenant);
            }
        });
    }
    /**
     * @param {?} tenant
     * @return {?}
     */
    addTenant(tenant) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            /** @type {?} */
            const tenants = (/** @type {?} */ ((yield this.tenantsStorage.read())));
            // Verificação de duplicidade de tenant
            if (tenants && tenants.length > 0) {
                tenants.forEach((/**
                 * @param {?} tenantFromStorage
                 * @param {?} index
                 * @return {?}
                 */
                (tenantFromStorage, index) => {
                    if (tenantFromStorage.shortNameTenant === tenant.shortNameTenant &&
                        tenantFromStorage.isDemo === tenant.isDemo) {
                        tenants.splice(index, 1);
                    }
                }));
            }
            this.tenantsStorage.data = tenants;
            this.tenantsStorage.data.push(tenant);
            this.sortTenantsByShortName();
            yield this.tenantsStorage.updateStorage();
            return tenant;
        });
    }
    /**
     * @param {?} tenant
     * @return {?}
     */
    deleteTenant(tenant) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready;
            try {
                yield this.tenantsStorage.delete(tenant.shortNameTenant);
                yield this.tenantsStorage.read();
            }
            catch (error) {
                console.error(error);
            }
        });
    }
    /**
     * @private
     * @return {?}
     */
    sortTenantsByShortName() {
        /** @type {?} */
        const sortBy = (/**
         * @param {?} fn
         * @return {?}
         */
        fn => (/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => -(fn(a) < fn(b)) || +(fn(a) > fn(b))));
        /** @type {?} */
        const getShortName = (/**
         * @param {?} o
         * @return {?}
         */
        o => o.shortNameTenant);
        /** @type {?} */
        const sortByShortName = sortBy(getShortName);
        this.tenantsStorage.data.sort(sortByShortName);
    }
}
MultiTenantService.SHORT_NAME_TENANT_FIELD = 'tenantId';
MultiTenantService.IS_DEMONSTRATION = 'isDemo';
MultiTenantService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
MultiTenantService.ctorParameters = () => [
    { type: Platform },
    { type: Storage },
    { type: Events }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MultiTenantModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: MultiTenantModule,
            providers: [
                MultiTenantService
            ],
        };
    }
}
MultiTenantModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FndModule, MockModule, asyncData, render, AuthModule, AuthService, DateTimeModule$1 as DateTimeModule, GlobalService, InterceptorModule, DefaultHttpInterceptorService, IntlLiteralsLoaderService, IntlModule, I18NPipe, IntlService, INTL_CONFIG, createTranslateLoader, ShowMessageModule, ShowMessageService, ShowLoadingService, ShowLoadingModule, MultiTenantEvents, MultiTenantService, MultiTenantModule, CameraUtilModule, CameraUtilEvents, CameraUtilProvider, FileUtilModule, FileUtilProvider, GeolocationUtilModule, GeolocationUtilProvider, HttpOauth2UtilModule, HttpOauth2UtilProvider, ImagePickerUtilEvents, ImagePickerUtilModule, ImagePickerUtilProvider, QRScannerUtilModule, QrScannerUtilProvider, SpeechRecognitionUtilModule, SpeechRecognitionUtilProvider, StorageUtil, SqLiteUtilBaseProvider, SqLiteUtilProvider, SqLiteUtilModule, SyncHandler, SyncEvents, SyncProvider, SYNC_CONFIG, SyncModule, SyncSqLiteUtil, OidcCordovaPopupWindow, OidcCordovaPopupNavigator, OidcCordovaIFrameNavigator, authFactory as ɵa, i18nFactory as ɵb, NavParamsMock as ɵc, PlatformMock as ɵd, SplashScreenMock as ɵe, StatusBarMock as ɵf };

//# sourceMappingURL=totvs-log-mobile-foundation.js.map